@@ -581,6 +581,7 @@ def get_internal_trades(api_client: ApiClient,
 def get_own_orders(api_client: ApiClient,
                    delivery_area: str = None,
                    portfolio_id: list[str] = None,
+                   contract_ids: Union[list[str], dict[str, str]] = None,
                    active_only: bool = False) -> list[OwnOrder]:
     """
     Function to collect all available Own Orders, either specific to portfolio and/or delivery area or all portfolios
@@ -590,6 +591,7 @@ def get_own_orders(api_client: ApiClient,
         api_client: PowerBot ApiClient
         delivery_area (str): EIC Area Code for Delivery Area
         portfolio_id (list[str]): List of specific portfolio IDs to load trades from. If left out, will load from all IDs.
+        contract_ids (list/dict): Collection of contract IDs to specifically load own orders for
         active_only (bool):  True if only active orders should be loaded. If False, loads also hibernate and inactive.
 
     Returns:
@@ -599,4 +601,5 @@ def get_own_orders(api_client: ApiClient,
                              data_type="own_order",
                              delivery_area=delivery_area,
                              portfolio_id=portfolio_id,
+                             contract_ids=contract_ids,
                              active_only=active_only)
