@@ -41,6 +41,7 @@ def get_orders(contract_hist_data: dict[str, pd.DataFrame],
                     if v and k in ["bid", "ask"]:
                         for x in v:
                             x["type"] = k
+                            x["order_id"] = str(x["order_id"])
                             x["best_bid"] = round(value.loc[nr, "best_bid_price"], 2)
                             x["best_bid_qty"] = round(value.loc[nr, "best_bid_quantity"], 2)
                             x["best_ask"] = round(value.loc[nr, "best_ask_price"], 2)
@@ -59,6 +60,7 @@ def get_orders(contract_hist_data: dict[str, pd.DataFrame],
                     if v and k in ["bids", "asks"]:
                         for x in v:
                             x["type"] = k[:-1]
+                            x["order_id"] = str(x["order_id"])
                             x["best_bid"] = round(value[nr].get("best_bid", 0), 2)
                             x["best_bid_qty"] = round(value[nr].get("best_bid_qty", 0), 2)
                             x["best_ask"] = round(value[nr].get("best_ask", 0), 2)
@@ -210,6 +212,7 @@ def get_orderbooks(contract_hist_data: dict[str, pd.DataFrame],
                    delivery_area: str = None,
                    timesteps: int = 15,
                    time_unit: str = "minutes",
+                   shortest_interval: bool = False,
                    timestamp: list[datetime] = None,
                    from_timestamp: bool = False,
                    api_client: Union[ApiClient, HistoryApiClient] = None,
@@ -287,7 +290,8 @@ def get_orderbooks(contract_hist_data: dict[str, pd.DataFrame],
         def process_multiple(contract_hist_data):
             for nr, (key, value) in enumerate(
                     tqdm(contract_hist_data.items(), desc="Processing Orders (multithreading)", position=0, unit="time periods", leave=False)):
-                _process_orderbook(key, value, str(new_dir), timesteps, time_unit, timestamp[nr] if timestamp else None, from_timestamp,
+                _process_orderbook(key, value, str(new_dir), shortest_interval, timesteps, time_unit, timestamp[nr] if timestamp else None,
+                                   from_timestamp,
                                    all_order_books, use_cached_data)
 
         # Define a splitter function that splits data into appropriately sized chunks without losing data
@@ -313,13 +317,14 @@ def get_orderbooks(contract_hist_data: dict[str, pd.DataFrame],
     else:
         # Processing Synchronously
         for nr, (key, value) in enumerate(tqdm(contract_hist_data.items(), desc="Processing Orders", unit="time periods", leave=False)):
-            _process_orderbook(key, value, str(new_dir), timesteps, time_unit, timestamp[nr] if timestamp else None, from_timestamp, all_order_books,
+            _process_orderbook(key, value, str(new_dir), shortest_interval, timesteps, time_unit, timestamp[nr] if timestamp else None,
+                               from_timestamp, all_order_books,
                                use_cached_data)
 
     # Cache Data as pickle
     if caching:
-        _cache_data("orderbook", all_order_books, delivery_area, exchange=exchange, api_client=api_client,
-                    gzip_files=False, timesteps=timesteps, time_unit=time_unit[0], as_json=False, as_pickle=True)
+        _cache_data("orderbook", all_order_books, delivery_area, exchange=exchange, api_client=api_client, gzip_files=False, timesteps=timesteps,
+                    time_unit=time_unit[0], shortest_interval=shortest_interval, as_json=False, as_pickle=True)
 
     # Saving Complete Orderbook as JSON
     if as_json:
@@ -482,11 +487,12 @@ def vwap_by_timeperiod(objects: pd.DataFrame,
         {45/30/15/0}T -> Execution To, determines the minutes that should be subtracted from Floor to reach end point for calculation
 
     Examples:
-        60T-60T-0T  <--> VWAP of the previous trading hour.
-        60T-15T-0T  <--> VWAP of the last quarter-hour of the previous trading hour.
-        60T-30T-15T <--> VWAP of third quarter-hour of the previous trading hour.
-        15T-60T-0T  <--> VWAP of last hour calculated from last quarter hour.
-        0T-60T-30T  <--> VWAP of first half of the last hour calculated from current timestamp.
+        Current Time: 16:23:30
+        60T-60T-0T  <--> VWAP of the previous trading hour (15:00-16:00).
+        60T-15T-0T  <--> VWAP of the last quarter-hour of the previous trading hour (15:45-16:00).
+        60T-30T-15T <--> VWAP of third quarter-hour of the previous trading hour (15:30-15:45).
+        15T-60T-0T  <--> VWAP of last hour calculated from last quarter hour (15:15-16:15).
+        0T-60T-30T  <--> VWAP of first half of the last hour calculated from current timestamp (15:23-15:53).
 
     Args:
         objects (pd.DataFrame): Collection of trades/ orders
@@ -549,3 +555,32 @@ def calc_rolling_vwap(trades: dict[str, pd.DataFrame],
         v[f"vwap_{t}"] = round(ptq_sum / quant_sum, 2).to_list()
 
     return trades
+
+
+def calc_orderbook_vwap(orderbook: dict[str, pd.DataFrame], depth: Union[int, float] = 0) -> tuple[pd.DataFrame, pd.DataFrame]:
+    """
+    Function to calculate value-weighted average prices for a single order book for bids and asks respectively.
+
+    Args:
+        orderbook (dict): Single order book
+        depth (int/float): Depth in MW to calculate average price for
+
+    Returns:
+        tuple(vwap_asks, vwap_bids)
+    """
+    # Add new column for cumulative quantity
+    asks = {k: df.insert(1, "cum_quant", value=df.quantity.cumsum()) or df for k, v in orderbook.items() if not (df := v.loc[v.type == "ask"].sort_values(by=['price'], ascending=True)).empty}
+    bids = {k: df.insert(1, "cum_quant", value=df.quantity.cumsum()) or df for k, v in orderbook.items() if not (df := v.loc[v.type == "bid"].sort_values(by=['price'], ascending=False)).empty}
+
+    # takes all orders up to desired depth or the first order if it exceeds depth or all orders if depth is 0
+    vwap_asks = pd.DataFrame({k: {
+        "vwap": round(sum((df := df_t if not (df_t := v.loc[v.cum_quant <= (depth if depth else max(v.cum_quant))]).empty else v.head(1)).price * df.quantity) / sum(
+            df.quantity), 2),
+        "depth": sum(v.quantity)} for k, v in asks.items()}).T
+
+    vwap_bids = pd.DataFrame({k: {
+        "vwap": round(sum((df := df_t if not (df_t := v.loc[v.cum_quant <= (depth if depth else max(v.cum_quant))]).empty else v.head(1)).price * df.quantity) / sum(
+            df.quantity), 2),
+        "depth": sum(v.quantity)} for k, v in bids.items()}).T
+
+    return vwap_asks, vwap_bids
