@@ -11,7 +11,7 @@ from urllib.parse import quote
 import pandas as pd
 import plotly.graph_objects as go
 from dateutil.tz import tzutc
-from powerbot_client import Signal, Trade, InternalTrade, OwnOrder
+from powerbot_client import Signal, Trade, InternalTrade, OwnOrder, ContractItem
 from pydantic import BaseModel, Field, validate_arguments, root_validator, validator
 from sqlalchemy import create_engine
 from sqlalchemy.exc import NoSuchModuleError, InterfaceError, ProgrammingError, OperationalError, TimeoutError
@@ -21,7 +21,7 @@ from tqdm import tqdm
 import powerbot_backtesting as pb
 from powerbot_backtesting.exceptions import SQLExporterError
 from powerbot_backtesting.utils.constants import *
-from powerbot_backtesting.utils.helper_functions import _cache_data, _find_cache, _get_file_cachepath
+from powerbot_backtesting.utils.helpers_general import _cache_data, _find_cache, _get_file_cachepath
 
 # Implement custom type for validation
 pandas_DataFrame = TypeVar('pandas.core.frame.DataFrame')
@@ -95,6 +95,7 @@ class BaseExporter(BaseModel, ABC):
                        delivery_area: str = None,
                        timesteps: int = 15,
                        time_unit: str = "minutes",
+                       shortest_interval: bool = False,
                        timestamp: list[datetime] = None,
                        from_timestamp: bool = False,
                        use_cached_data: bool = True,
@@ -133,6 +134,7 @@ class BaseExporter(BaseModel, ABC):
                                  delivery_area=self.delivery_area if hasattr(self, 'delivery_area') and not delivery_area else delivery_area,
                                  timesteps=timesteps,
                                  time_unit=time_unit,
+                                 shortest_interval=shortest_interval,
                                  timestamp=timestamp,
                                  from_timestamp=from_timestamp,
                                  api_client=self.client if hasattr(self, 'client') else None,
@@ -242,6 +244,21 @@ class BaseExporter(BaseModel, ABC):
                                     rolling_interval=rolling_interval,
                                     rolling_time_unit=rolling_time_unit)
 
+    @staticmethod
+    @validate_arguments
+    def calc_orderbook_vwap(orderbook: dict[str, pandas_DataFrame], depth: Union[int, float] = 0) -> tuple[pandas_DataFrame, pandas_DataFrame]:
+        """
+        Function to calculate value-weighted average prices for a single order book for bids and asks respectively.
+
+        Args:
+            orderbook (dict): Single order book
+            depth (int/float): Depth in MW to calculate average price for
+
+        Returns:
+            tuple(vwap_asks, vwap_bids)
+        """
+        return pb.calc_orderbook_vwap(orderbook=orderbook, depth=depth)
+
     @staticmethod
     @validate_arguments
     def plot_ohlc(ohlc_data: dict[str, pandas_DataFrame],
@@ -348,10 +365,9 @@ class ApiExporter(BaseExporter):
         api_key, host = values.get('api_key'), values.get('host')
 
         pattern_key = "\w{8}-\w{4}-\w{4}-\w{4}-\w{12}"
-        pattern_host = "https://\w{4,7}.powerbot-trading.com/\w+/\w+/v\d{1}/api"
+        pattern_host = "https://\w{4,7}.powerbot-trading.com(:443)?/\w+/\w+/v\d{1}/api"
 
-        if not re.match(pattern_key, api_key) or not re.match(pattern_host, host):
-            raise ValueError("Your credentials do not conform to the necessary formats")
+        assert re.match(pattern_key, api_key) and re.match(pattern_host, host), "Your credentials do not conform to the necessary formats"
 
         return values
 
@@ -363,7 +379,7 @@ class ApiExporter(BaseExporter):
                       contract_time: str = "all",
                       products: list[str] = None,
                       allow_udc: bool = False,
-                      delivery_areas: list[str] = None) -> list[dict]:
+                      delivery_areas: list[str] = None) -> list[ContractItem]:
         """
         Loads all contracts for specified timeframe. Alternatively, a list of contract IDs can be passed instead of a timeframe.
 
@@ -377,7 +393,7 @@ class ApiExporter(BaseExporter):
             delivery_areas (list): List of EIC-codes
 
         Returns:
-            dict{key: (list[str])}: Dictionary of Lists containing Contracts
+            list[ContractItem]: List containing Contracts
         """
         return pb.get_contracts(api_client=self.client,
                                 time_from=time_from,
@@ -607,6 +623,7 @@ class ApiExporter(BaseExporter):
     def get_own_orders(self,
                        delivery_area: str = None,
                        portfolio_id: list[str] = None,
+                       contract_ids: Union[list[str], dict[str, list[str]]] = None,
                        active_only: bool = False) -> list[OwnOrder]:
         """
         Function to collect all available Own Orders, either specific to portfolio and/or delivery area or all portfolios
@@ -615,6 +632,7 @@ class ApiExporter(BaseExporter):
         Args:
             delivery_area (str): EIC Area Code for Delivery Area
             portfolio_id (list[str]): List of specific portfolio IDs to load trades from. If left out, will load from all IDs.
+            contract_ids (list/dict): Collection of contract IDs to specifically load own orders for
             active_only (bool):  True if only active orders should be loaded. If False, loads also hibernate and inactive.
 
         Returns:
@@ -623,6 +641,7 @@ class ApiExporter(BaseExporter):
         return pb.get_own_orders(api_client=self.client,
                                  delivery_area=delivery_area,
                                  portfolio_id=portfolio_id,
+                                 contract_ids=contract_ids,
                                  active_only=active_only)
 
     @validate_arguments
@@ -851,10 +870,12 @@ class SQLExporter(BaseExporter):
     Instructions for passing arguments to exporter functions:
         - String
             If keyword argument is a string, it will be simply compared with '='. Optionally, a mathematical/ SQL
-            operator (LIKE, <, <=, >, >=, <>) can be passed within the string. This operator will be used instead of '='.
+            operator (LIKE, BETWEEN x AND y, <, <=, >, >=, <>) can be passed within the string. This operator will be used instead of '='.
 
             Example:
-                best_bid='> 0.00' -> best_bid > 0.00 / as_of="> 2020-09-20 00:00:00" -> as_of > '2020-09-20 00:00:00'
+                best_bid='> 0.00' -> best_bid > 0.00
+                as_of="> 2020-09-20 00:00:00" -> as_of > '2020-09-20 00:00:00'
+                exec_time='BETWEEN 2021-09-10 AND 2021-09-11' -> exec_time BETWEEN '2021-09-10' AND '2021-09-11'
 
         - Tuple
             If keyword argument is a tuple, it will be checked, if parameter is one of the elements of the tuple.
@@ -1166,8 +1187,9 @@ class SQLExporter(BaseExporter):
         if as_dataframe and len(delivery_area) > 1:
             raise ValueError("Please only give one delivery area when loading data as dataframes")
 
+        trades = {}
+
         if use_cached_data and as_dataframe and (contract_ids := kwargs.get("contract_id")):
-            trades = {}
             missing_contracts = {}
 
             # Find __cache__ directory
@@ -1205,7 +1227,7 @@ class SQLExporter(BaseExporter):
 
             if delivery_area:
                 for i in delivery_area:
-                    sql_params += f" AND (buy_delivery_area = '{i}' OR sell_delivery_area = '{i}')"
+                    sql_params += f" {'AND' if 'WHERE' in sql_params else 'WHERE'} (buy_delivery_area = '{i}' OR sell_delivery_area = '{i}')"
 
             with self.__get_session() as session:
                 if not (result := session.execute(f"SELECT * FROM public_trades{sql_params}").fetchall()):
@@ -1617,6 +1639,8 @@ class SQLExporter(BaseExporter):
         Returns:
 
         """
+        self.engine
+
         with self.__get_session() as session:
             try:
                 result = session.execute(sql_statement).fetchall()
@@ -1640,7 +1664,7 @@ class SQLExporter(BaseExporter):
         if not all(arg for arg in kwargs.values()):
             raise SQLExporterError("Some of your input values are invalid or empty!")
         sql_params = ""
-        operators = ["LIKE", "<", "<=", ">", ">=", "<>"]
+        operators = ["LIKE", "BETWEEN", "<", "<=", ">", ">=", "<>"]
 
         for keyword, argument in kwargs.items():
             op = "="
@@ -1659,6 +1683,11 @@ class SQLExporter(BaseExporter):
                             op, argument = argument.split(" ")
                         if op == "LIKE":
                             argument = f"%{argument}%"
+                        if op == "BETWEEN":
+                            if " AND " not in argument:
+                                raise SQLExporterError(f"Your input for {keyword} does not conform to the guidelines. Please revise")
+                            a1, a2 = argument.split(" AND ")
+                            argument = f"'{a1}' AND '{a2}'"
                         try:
                             datetime.strptime(argument, DATE_YMD_TIME_HMS)
                         except:
@@ -1695,7 +1724,10 @@ class SQLExporter(BaseExporter):
                         argument = float(argument)
                     sql_params += f" {sql_statement} {keyword} {op} {argument}"
                 except:
-                    sql_params += f" {sql_statement} {keyword} {op} '{argument}'"
+                    if isinstance(argument, str) and len(argument.split(" AND ")) > 1:
+                        sql_params += f" {sql_statement} {keyword} {op} {argument}"
+                    else:
+                        sql_params += f" {sql_statement} {keyword} {op} '{argument}'"
 
         return sql_params
 
